/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1998-1999 Jeremie Miller.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 * --------------------------------------------------------------------------*/
#include "jsm.h"

/* LDAP Authentication Module for jabber  */
/* Author : Amit Hooja                    */
/* Date   :  27th October 2001            */
/* Email  : amithooja@datainfosys.net , java@datainfosys.net , a_hooja@hotmail.com */
/* ------------------------------------------------ */

#include <stdio.h>
#include <ldap.h>
#include <ldap_ssl.h>
#include <string.h>

/* modify the HOSTNAME and PORTNUMBER poitning to ur ldap server */
#define HOSTNAME "202.157.19.1"
#define PORTNUMBER 1289
#define LDAPCHK_SUCCESS 1 /* TRUE */ 
#define LDAPCHK_FAIL    0 /* FALSE */
#define Z_DEBUG
/* ---------------------------------------- */

mreturn ldap_auth(mapi m, void *arg)
{
   short cause;
   char *in_pw;
   char *in_un;
   char uname[36];
	
   fprintf(stderr,"ldap auth control ..........\n\n");
   log_debug("mod_auth_ldap_ereg","ldap auth control ... ");

   if (m->packet->type != JPACKET_IQ) 
	return M_IGNORE;
		
	if(jpacket_subtype(m->packet) == JPACKET__GET)
   { /* type=get means we flag that the server can do digest auth */
      xmlnode_insert_tag(m->packet->iq,"password");
      return M_PASS;
   }



   if (jpacket_subtype(m->packet) != JPACKET__SET)
      return M_PASS;
		

   if ((in_un = xmlnode_get_tag_data(m->packet->iq,"username")) == NULL) 
   {
	jutil_error(m->packet->x, TERROR_AUTH);
	return M_HANDLED;
   }
		
   if ((in_pw = xmlnode_get_tag_data(m->packet->iq,"password")) == NULL) 
   {
	jutil_error(m->packet->x, TERROR_AUTH);
	return M_HANDLED;		
   }


	
   strcpy(uname, in_un);
	
   fprintf(stderr,"chking .....%s....\n\n",uname);

   if (auth(uname, in_pw) == LDAPCHK_SUCCESS)
   {
	log_debug("mod_auth_ldap_eaut","LDAP OK cause=%d", cause);
	fprintf(stderr,"ldap okie ");	
	jutil_iqresult(m->packet->x);
   }
   else
   {	
	log_debug("mod_auth_ldap_eaut","LDAP NOT FOUND cause=%d", cause);
	jutil_error(m->packet->x, TERROR_AUTH);	
   }

   return M_HANDLED;   
}


mreturn ldap_noreg(mapi m, void *arg)
{
   
	if (m->packet->type != JPACKET_IQ) 
		return M_IGNORE;
	
	log_debug("mod_auth_ldap_ereg","ldap NO Registration ... ");
		
	if (jpacket_subtype(m->packet) != JPACKET__SET) return M_PASS;

	jutil_error(m->packet->x, TERROR_FORBIDDEN);
	
	return M_HANDLED;
}


/* the main startup/initialization function */
void mod_auth_ldap(jsmi si)
{
	fprintf(stderr,"inside ldap module----> \n");
	printf("ldap modules");
	js_mapi_register(si, e_REGISTER, ldap_noreg, NULL);
	js_mapi_register(si, e_AUTH, ldap_auth, NULL); 
	fprintf(stderr,"registration of functions complete \n ");
}


/* authorize user using ldap simple authentication */
int auth(char *user , char *passwd)
{

LDAP                  *ld;
LDAPMessage                  *res;
struct timeval                  zerotime;
char *bindstring;
char *bindpw;
int irs;



bindstring=(char *) malloc(36);
bindpw=(char *) malloc(36);


zerotime.tv_sec = zerotime.tv_usec = 0L;


if ( (ld = ldap_init( HOSTNAME, PORTNUMBER )) == NULL ) {
   printf( "Error in initializing ldap" );
   fprintf(stderr,"error inititalizing ldap\n\n");

   return( 1 );
}


strcpy(bindstring,"uid=\0");
strcat(bindstring,user);
strcat(bindstring,", o=ta.net\0");

bindpw=strcpy(bindpw,passwd);
 fprintf(stderr,"setup atuhs -amit hooja inititalizing ldap\n\n");
fprintf(stderr,"---------%s --------------%s-- \n\n",bindstring,bindpw);

irs =  ldap_simple_bind_s( ld, bindstring,bindpw);


if (irs != 0 ) {
   printf(  "\nAuthentication failed:\n " );
   fprintf(stderr,"authencticatin fails ------%d---->",irs);
    
   return( 0 );
}


fprintf(stderr,"out of if of auth");
ldap_unbind( ld );
fprintf(stderr, "auth ok \n");

return( 1 );
}


